var fs = require('fs');
var app = require('app');
var path = require('path');
var BrowserWindow = require('browser-window');

// Limitation:
// Some linux computers with certain intel and older GPUs, webviews
// tend to fall to render certain pages, so we need to disable gpu.
if (process.platform === 'linux') {
  app.commandLine.appendSwitch('disable-gpu');
}

// Note
// We will keep some special data in storage.json to pre-configure
// the window or the other details.
var storageDataPath = path.join(app.getPath('userData'), 'storage.json');
var storageData;

try {
  storageData = JSON.parse(fs.readFileSync(storageDataPath, 'utf8'));
} catch(e) {
  storageData = {};
}

// Report crashes to our server.
require('crash-reporter').start();

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
var mainWindow = null;

// Quit when all windows are closed.
app.on('window-all-closed', function() {
  // On OS X it is common for applications and their menu bar
  // to stay active until the user quits explicitly with Cmd + Q
  if (process.platform != 'darwin') {
    app.quit();
  }
});

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
app.on('ready', function() {
  var windowOptions = {
    'title': 'Vectr',
    'frame': false,
    'resizable': true,
    'min-width' : 1000,
    'min-height' : 600,
    'width': 1000,
    'height': 600
  };

  var windowBoundsInStorage = storageData['window.bounds'];
  if (windowBoundsInStorage) {
    var keys = ['x', 'y', 'width', 'height'];
    keys.forEach(function(eachKey) {
      windowOptions[eachKey] = windowBoundsInStorage[eachKey];
    })
  }

  mainWindow = new BrowserWindow(windowOptions);

  mainWindow.webContents.session.setDownloadPath(app.getPath('userDesktop'));

  mainWindow.on('close', function() {
    storeData();
  });

  // Emitted when the window is closed.
  mainWindow.on('closed', function() {
    // Dereference the window object, usually you would store windows
    // in an array if your app supports multi windows, this is the time
    // when you should delete the corresponding element.
    mainWindow = null;
  });

  // We need to add `will-navigate` handler to make sure the browser won't
  // open the file when users drop them into the application.
  //
  // Reference: https://goo.gl/U3vH5R
  mainWindow.webContents.on('will-navigate', function(e) {
    e.preventDefault();
  });

  mainWindow.loadUrl('file://' + __dirname + '/index.html');
});

function storeData() {
  // keep window bounds when the window is going to be closed
  var bounds = mainWindow.getBounds();
  storageData['window.bounds'] = bounds;
  fs.writeFileSync(storageDataPath, JSON.stringify(storageData));
}
